#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> click.tcl                                               <--#
#-->                                                         <--#
#--> This tcl file handles the direct typing of the          <--#
#--> parameters   into the control window.                   <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc click arg {
    global env sortfilename directory mydebug
    global file_status data_type
    global numitapes imedianum simedianame imediavals
    global bfmt atext dtext ffmt midasbase
    global debugsort
    global overwritespec
    global debuglevel 
    global configtable
    global update update_time
    global vdcurrent_pos vdaspos vdpos
    global uv1 uv2 uv3 uv4 uv5 uv6 uv7 uv8 uv9 uv10 uv11 uv12
    global uv13 uv14 uv15 uv16
    global csdev0value
    global shmstrings shmvalue shmconstant

    set f [lindex $arg 0]
    set c [lindex $arg 1]
    set w [lindex $arg 2]
    set v [lindex $arg 3]
    set i [lindex $arg 4]

    if { $mydebug == 1 } then {
	puts " f $f "
	puts " c $c "
	puts " w $w "
	puts " v $v "
	puts " i $i "
    }

    switch $w {
	TSORTFILE        { tsortfile $v }
	BBROWSE          { browse_sortfile }
	BEDITFILE        { edit_xlate }
	BCOMPILE         { compile_sortfile }
	BSOPTS           { sort_options }
	CSMEDIA          { input_media $v }
	CSFMT            { set data_type $v }
	BIMEDIADATA      { tape_info }
	BBROWSEDEV       { input_browse_device }
	BCFTAB           { user_var }
	CSMEDIA0         { }
	CSFMTO           { }
	TDEVICEO         { }
	BBROWSEDEVO      { }
	MSPEC            { }
	TSAVEDIR         { }
	NREFRESH         { }
        BSPECTRA         { inform sb sfmstart localsas }
	CSOS             { set overwritespec $v }
	BSTART           { lets_go }
	BSTOP            { lets_not_go }
	BQUIT            { leave_sort }
	BSVARS           { }
	BSETTINGS        { get_settings }
	DCLIST           { in_list $v }
	NIMEDIATAPES     { set numitapes $v }
	NIMEDIAVAL       { set imedianum $v }
	TIMEDIANAME      { set simedianame $v }
	BIMEDIAADD       { add-imedia }
	BIMEDIAREMOVE    { remove-imedia }
	BIMEDIAOK        { ok-imedia }
	IMEDIALIST       { choice-imedia $i }
	CSMEDIAD         { set ffmt $v }
	NIMEDIADTAPES    { set numitapes $v }
	TTEXTD1          { set dtext $v }
	TTEXTD2          { set atext $v }
	CSMEDIADLEN      { set bfmt $i }
	IMEDIADLIST      { choice-imediad $i }
	BIMEDIADADD      { add-imediad }
	BIMEDIADREMOVE   { remove-imediad }
	BIMEDIADGENERATE { gen-imediad }
	BMEDIADSET       { ok-imediad }	
	BIMEDIADCLEAR    { clear-imediad }	
	BIMEDIADRD       { paint-imediadlist }
	TSETMBDIR        { set midasbase $v }
	TSETDIR          { set_directory $v }
	TSETDATADIR      { set_sortdatadir $v }
	TSETGAINDIR      { set_gaindirectory $v }
	TSETGATEDIR      { set_gatedirectory $v }
	TSETUM           { set_makefilepath $v }
	TSETTAB          { set_configtable $v }
	CBUSEGCC         { set_usegcc $v }
	CBSSSNC          { set_usesssnc $v }
	CBKMSAS          { set_kmemsas $v }
	BMEMSASCTRL      { memsas_ctrl }
	BMEMSAS1         { memsas_ctrl.kill_daemon }
	BMEMSAS2         { memsas_ctrl.leave }
	BMEMSAS3         { memsas_ctrl.refresh }
	LMEMSAS1         { memsas_ctrl.select $i $v }
	BSETREDISPLAY    { gs-redisplay }
	CSDSO            { debug_level $v }
	BDSO             { debug_options }
	BVDATA           { view_data }
	CSVD1            { set vdpos $v ; viewData.redisplay }
	CSVD2            { set vdaspos $v ; viewData.redisplay }
	BVD1             { viewData.cancel }
	BVD2             { viewData.redisplay }
	CBTO             { enable_set_trace $v }
	CSTO             { set_trace_level $i }
	CBDOALL          { setalld $v }
	CBDOU0           { setuserd $v }
	CBDOU1           { set debuglevel(user1) $v }
	CBDOU2           { set debuglevel(user2) $v }
	CBDOU3           { set debuglevel(user3) $v }
	CBDOU4           { set debuglevel(user4) $v }
	CBDOU5           { set debuglevel(user5) $v }
	CBDOU6           { set debuglevel(user6) $v }
	CBDOU7           { set debuglevel(user7) $v }
	CBDOU8           { set debuglevel(user8) $v }
	CBDOU9           { set debuglevel(user9) $v }
	CBDOL0           { setlibd $v }
	CBDOL1           { set debuglevel(inc1d) $v }
	CBDOL2           { set debuglevel(inc2d) $v }
	CBDOL3           { set debuglevel(ucid) $v }
	CBDOL4           { set debuglevel(ugm) $v }
	CBDOL5           { set debuglevel(hist) $v }
	CBDOMS0          { setmaind $v }
	CBDOMS1          { set debuglevel(spectra) $v }
	CBDOMS2          { set debuglevel(end_prog) $v }
	CBDOMS3          { set debuglevel(decipher) $v }
	CBDOMS4          { set debuglevel(get_block) $v }
	CBDOMS5          { set debuglevel(parse_args) $v }
	CBDOMS6          { set debuglevel(main) $v }
	CBDOMS7          { set debuglevel(shared) $v }
	CBDUI            { set debuglevel(uinteract) $v }
	CBTB             { set debuglevel(tback) $v }
	CBGAT            { set debuglevel(gate) $v }
	BDOSO            { save_debug_levels }
	CBTHIST          { setthist $v }
	CSDISK           { csdisk $v }
	LDISK            { select-item $i }
	NDISK            { ndisk $v }
	NDISK1           { ndisk1 $v }
	CSDISK1          { set bfmt $i }
	TDISK            { set dtext $v }
	TDISK1           { set atext $v }
	BDISK6           { browse_disk_files }
	BDISK0           { add-list }
	BDISK1           { paint-disklist }
	BDISK2           { remove-list }
	BDISK3           { clear-disklist }
	BDISK4           { save-diskvalues }
	BDISK5           { destroy-frame FDISK }
	CSDDSO           { set update $v ; set update_time [ expr $i * 1000 ] }
	TUV1             { set uv1 $v }
	TUV2             { set uv2 $v }
	TUV3             { set uv3 $v }
	TUV4             { set uv4 $v }
	TUV5             { set uv5 $v }
	TUV6             { set uv6 $v }
	TUV7             { set uv7 $v }
	TUV8             { set uv8 $v }
	TUV9             { set uv9 $v }
	TUV10            { set uv10 $v }
	TUV11            { set uv11 $v }
	TUV12            { set uv12 $v }
	TUV13            { set uv13 $v }
	TUV14            { set uv14 $v }
	TUV15            { set uv15 $v }
	TUV16            { set uv16 $v }
	BUV1             { user_var.write }
	CBDEV0           { set mydebug $v }
	CSDEV0           { set csdev0value $v }
	CSDEV1           { set shmvalue $v }
	NDEV0            { set shmconstant $v }
	BDEV0            { run_shmctrl $csdev0value $shmvalue $shmconstant }
    }

}


#	BSTATUS          { status }
