#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
#--> view_data.tcl : Launced from sort options. Allows the   <--#
#-->                 user to view their raw data that is     <--#
#-->                 being processed by the sort program.    <--#
#-->                                                         <--#
#--> D.Appelbe Daresbury NPG 01/05/04                        <--#
#-->                                                         <--#
#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc view_data {} {
    global mydebug
    global running
    #--> Create the GUI
    viewData.createFrame
    #--> Configure the tags that we shall use ....
    viewData.configure_tags
    #--> If we are running lets display some data
    if { $mydebug == 1 } then { puts "viewData : running = $running" }
    if { $running == 1 } then {
	viewData.redisplay
    }
    #--> Else it is up to the user to sort things out
}

#--> +++++++++++++++++++++++++++++++++++++++++++++++++++++++ <--#
proc viewData.createFrame {} {
    global mydebug
    global vdcurrent_pos
    global vdpos
    global vdaspos
    global vdpostrue

    set vdcurrent_pos "Current position in block is : 0"
    set vals        [ list "From start" \
			   "Current -1024" "Current -512" "Current -256" \
			   "Current -128" "Current -64" "Current -32" \
			   "Current position" \
			   "Current +32" "Current +64" "Current +128" \
			   "Current +256" "Current +512" "Current +1024" \
			   "Range +/- 1024" "Range +/- 512" "Range +/- 256" \
			   "Range +/- 128" "Range +/- 64" "Range +/- 32"
			 ]
    set vdpostrue   [ list 0 -1024 -512 -256 -128 -64 -32 -1 32 64 128 256 512 1024 \
			   10024 9512 9256 9128 9064 9032 ]
    set asvals      [ list "8bit" "16bit" "32bit" ]

    if { $mydebug == 1 } then { puts "viewData.createFrame : In Routine" }

    :frame FVIEWDATA :label "MIDASsort: Data Viewer" :resize F :transient T
     :panel PVIEWDATA :layout V :padx 10 :pady 10
      :layout H
      :message MVD1 :label $vdcurrent_pos :bold T
      :next-row T
      :galley GVD1 :cols 100 :rows 40 :deaf T :font fixed :see 0 :scroll V
      :next-row T
      :choice-stack CSVD1 :label "View from" :strings $vals :value $vdpos
      :hskip 40
      :choice-stack CSVD2 :label "View as" :strings $asvals :value $vdaspos
      :hskip 20
      :button BVD2 :label "Show"
      :hskip 20
      :button BVD1 :label "Cancel"
    
    :show T

    if { $mydebug == 1 } then { puts "viewData.createFrame : Leaving Routine" }
}

proc viewData.cancel {} {
    destroy-frame FVIEWDATA
}

proc viewData.redisplay {} {
    global mydebug

    if { $mydebug == 1 } then { puts "viewData.redisplay : Entering Routine" }
    #--> First run the data format program
    set arg1 [ viewData.setrange ]
    set arg2 [ viewData.setbits ]
    set arg3 "r"
    if { $mydebug == 1 } then { puts "viewData.redisplay : range -> $arg1" }
    if { $mydebug == 1 } then { puts "viewData.redisplay : bits  -> $arg2" }

    set contents ""
    set z [ catch { exec "midasSortfb" $arg1 $arg2 $arg3 } contents ]
    if { $z != 0 } then { midas-error "Could not run midasSortfb" }
    if { $mydebug == 1 } then { puts "viewData.redisplay : z  -> $z" }

    #--> Now read the data from the disc file
    set z [ catch { open "msdata.txt" r } f ]
    if { $z == 0 } then {
	gets $f m
	close $f
    } else {
	if { $mydebug == 1 } then { puts "viewData.redisplay : Could not open msdata.txt" }
    }

    set vdcurrent_pos "Current position in block is : $m"
    :modify MVD1 :label $vdcurrent_pos

    #--> So here we do something semi nifty so that it is easy to see
    #--> the data word corresponding ground zero
    #--> First which line of the data will it be on?
    set start [ viewData.getstart $arg1 $arg2 $m ]
    set stop  [ expr [ viewData.getstop $arg1 $arg2 $m ] + 1 ]
    if { $mydebug == 1 } then { puts "viewData.redisplay : start = $start" }
    if { $mydebug == 1 } then { puts "viewData.redisplay : stop = $stop" }
    #--> So there are 8 data wrds per line so which line are we interested in?
    set numlines [ expr [ expr $stop - $start ] / 8 ]
    if { $arg2 == 32 } then { set numlines [ expr $numlines / 2 ] }
    if { $numlines == 0 } then { set numlines 1 }
    if { $mydebug == 1 } then { puts "viewData.redisplay : numlines = $numlines" }
    if { $arg1 == -1 } then {
	set line 1
	set wrd  2
    }

    if { $arg1 > 0 } then {
	if { $arg1 < 9000 } then {
	    set line 1
	    set wrd 1
	} else {
	    set line [ expr [ expr $numlines / 2 ] + 1 ]
	    set wrd 1
	}
    } 
    if { $arg1 < -1 } then {
	set line $numlines
	set wrd 8
    }

    set mywrd [ expr [ expr [ expr [ expr [ expr $line - 1 ] * 8 ] + $wrd ] + 6 ] + [ expr $line * 2 ] ]

    if { $mydebug == 1 } then { puts "viewData.redisplay : arg1 = $arg1" }
    if { $mydebug == 1 } then { puts "viewData.redisplay : line = $line" }
    if { $mydebug == 1 } then { puts "viewData.redisplay : wrd  = $wrd" }
    if { $mydebug == 1 } then { puts "viewData.redisplay : mywrd= $mywrd" }

    :modify GVD1 :empty T
    set wrdctr 1
    set date 1
    set posctr 1
    foreach w $contents {
	if { $mydebug == 1 } then { puts "wrd($wrdctr/$posctr) = $w" }
	if { $date == 1 } then {
	    :modify GVD1 :usingtag tagbase :strings $w
	    :modify GVD1 :usingtag tagbase :strings " "
	    set posctr [ expr $posctr + 1 ]
	    if { $posctr == 7 } then {
		set date 0
		set posctr 1
		:modify GVD1 :usingtag tagbase :strings "\n"
	    }
	} else {
	    if { $posctr > 2 } then {
		if { $wrdctr == $mywrd } then {
		    :modify GVD1 :usingtag tagstart :strings [ format "%-9s" $w ]
		    :modify GVD1 :usingtag tagstart :strings " "
		} else {
		    :modify GVD1 :usingtag tagbase :strings [ format "%-9s" $w ]
		    :modify GVD1 :usingtag tagbase :strings " "
		}
	    } else {
		:modify GVD1 :usingtag tagbase :strings [ format "%6s" $w ]
		:modify GVD1 :usingtag tagbase :strings " "
	    }
	    set posctr [ expr $posctr + 1 ]
	    if { $posctr == 11 } then {
		set posctr 1
		:modify GVD1 :usingtag tagbase :strings "\n"
	    }	    
	}

	set wrdctr [ expr $wrdctr + 1 ]
    }


    if { $mydebug == 1 } then { puts "viewData.redisplay : Leaving Routine" }
}

proc viewData.setrange {} {
    global vdpos
    global vdpostrue

    return [ lindex $vdpostrue $vdpos ]
}

proc viewData.setbits {} {
    global vdaspos

    switch $vdaspos {
	0         { return  8 }
	1         { return 16 }
	2         { return 32 }
    }

}

proc viewData.configure_tags {} {
    global mydebug

    if { $mydebug == 1 } then { puts "viewData.configure_tags : Entered" }
    :modify GVD1 :createtag "tagbase  -foreground blue"
    :modify GVD1 :createtag "tagstart -foreground red"
    if { $mydebug == 1 } then { puts "viewData.configure_tags : Leaving" }

}

proc viewData.getstart args {
    global mydebug

    if { $mydebug == 1 } then { puts "viewData.getstart: In Routine" }
    set range [ first $args ]
    if { $mydebug == 1 } then { puts "viewData.getstart: Range = $range" }
    set bits  [ second $args ]
    if { $mydebug == 1 } then { puts "viewData.getstart: bits = $bits" }
    set base  [ third $args ]
    if { $mydebug == 1 } then { puts "viewData.getstart: base = $base" }

    if { $range < 0 } then {
	set tmp [ expr $base + $range ]
	if { $tmp > -1 } then {
	    set start $tmp
	} else {
	    set start 0
	}
	if { $mydebug == 1 } then { puts "viewData.getstart: start = $start" }
    } 

    if { $range > 0 } then {
	if { $range < 9000 } then {
	    set start $base
	} else {
	    set range [ expr $range - 9000 ]
	    set tmp [ expr $base - $range ]
	    if { $tmp > -1 } then {
		set start $tmp
	    } else {
		set start 0
	    }
	}
    }

    if { $bits == 32 } then {
	set start [ expr $start / 2 ]
	set start [ expr $start * 2 ]
    }

    if { $mydebug == 1 } then { puts "viewData.getstart: start = $start" }
    if { $mydebug == 1 } then { puts "viewData.getstart: Out of Routine" }
    return $start
}

proc viewData.getstop args {
    global mydebug

    if { $mydebug == 1 } then { puts "viewData.getstop: In Routine" }
    set range [ first $args ]
    if { $mydebug == 1 } then { puts "viewData.getstop: Range = $range" }
    set bits  [ second $args ]
    if { $mydebug == 1 } then { puts "viewData.getstop: bits = $bits" }
    set base  [ third $args ]
    if { $mydebug == 1 } then { puts "viewData.getstop: base = $base" }

    if { $range < 0 } then {
	set stop $base
	if { $mydebug == 1 } then { puts "viewData.getstop: stop = $stop" }
    } 

    if { $range > 0 } then {
	if { $range < 9000 } then {
	    set tmp [ expr $base + $range ]
	    if { $tmp < 32768 } then {
		set stop $tmp
	    } else {
		set stop 16384
	    }
	} else {
	    set range [ expr $range - 9000 ]
	    set tmp [ expr $base + $range ]
	    if { $tmp < 32768 } then {
		set stop $tmp
	    } else {
		set stop 16384
	    }
	}
    }

    if { $range == -1 } then {
	set stop [ expr $base + 7 ]
    }

    if { $bits == 32 } then {
	set stop [ expr $stop / 2 ]
	set stop [ expr $stop * 2 ]
    }

    if { $mydebug == 1 } then { puts "viewData.getstop: stop = $stop" }
    if { $mydebug == 1 } then { puts "viewData.getstop: Out of Routine" }
    return $stop
}