# cloverge.tcl     clover card Ge channel control

#   March 25 1996    conversion for tk
# Wed Nov 30 11:02:18 GMT 1994, DB
#   printing via print templates introduced
# 23-12-93 PHO
#	corrected presets menue
# 27-7-93 PHO
#	created file from ge.tcl and adapted for clover card

set debug 0
set started 0

set frame CLOVERGE
set frametitle "Clover Ge Channel Setup"
set icontitle "CloverGe"

set chanlist {???}
set chanindex 0
set seglist {A B C D E}
set segindex 0

set defaultchanlist {0 1 2 3 4 5 6 7}

set cardtype Clover
set chantype Ge
set VXIGlobal Ge


#    The array "table" which is indexed by the "widget name" contains all the information needed
#    for the register/display item
#
#    Each element of "table" is a list
#
#       0.    name of global parameter used to hold current value of the register
#       1.    procedure to be used to convert the widget value into parameter value
#       2.    procedure to be used to convert the parameter value into a widget value
#       3.    name of the register
#       4.    procedure to be used to convert the register value into parameter value
#       5.    procedure to be used to convert the parameter value into a register value
#       6.    name of unit used for register access
#       7.    widget type to be used for "by parameter" displays
#       8-10  parameters for the procedure creating "by parameter" displays
#              eg  if (7) is :slider then 8 is min value, 9 is max value and 10 is unit
#       11    for startup only - initial value of the parameter (1)
#       12    for startup only - inserted for this widget into the table "parameternames"


set table(ALLCHANS) {allchans int-widget->param param->int-widget nil nil nil nil nil nil nil nil 0 nil}
set table(ALLSEGS)  {allsegs  int-widget->param param->int-widget nil nil nil nil nil nil nil nil 0 nil}
set table(TFA) {tfagain int-widget->param param->int-widget "TFAGain"
       int-reg->param param->int-reg nil :choice Low High nil 1 "TFA Gain"}
set table(MODE) {mode int-widget->param param->int-widget "TestMode"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Test Mode"}
set table(ENABLE) {enable int-widget->param param->int-widget "Enable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "Enable"}
set table(CONFENETRES) {confenetres int-widget->param param->int-widget "ConFenetres"
       int-reg->param param->int-reg nil :checkbox-c nil nil nil 1 "Con Fenetres"}
set table(LASTCARD) {lastcard int-widget->param param->int-widget "LastCard"
       int-reg->param param->int-reg nil :checkbox-c nil nil nil 1 "Last Card"}
set table(MARKREJECT) {markreject int-widget->param param->int-widget "PileupReject"
       int-reg->param param->int-reg nil :choice-c Mark Reject nil 1 "Pileup"}
set CFDTHRESHLOW {cfdthresh slider-widget->param param->int-widget "CFD.Thresh"
       unit-reg->param param->unit-reg keV :slider 0 2600 1 1 "CFD Threshold (keV)"}
set CFDTHRESHHIGH {cfdthresh slider-widget->param param->int-widget "CFD.Thresh"
       unit-reg->param param->unit-reg keV :slider 0 825 1 1 "CFD Threshold (keV)"}
set table(CFDTHRESH) $CFDTHRESHHIGH
set table(CFDDELAY) {cfddelay slider-widget->param param->int-widget "CFD.Delay"
       unit-reg->param param->unit-reg ns :slider 0 200 1 1 "CFD Delay (ns)"}
set table(CFDWIDTH) {cfdwidth slider-widget->param param->int-widget "CFD.Width"
       unit-reg->param param->unit-reg ns :slider-c 0 200 1 1 "CFD Width (ns)"}
set table(POLEZERO) {polezero slider-widget->param param->int-widget "PoleZero"
       int-reg->param param->int-reg nil :slider 0 255 1 1 "Pole Zero"}
set table(THRESH20) {thresh20 int-widget->param param->int-widget "20MeV.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "20 MeV Threshold"}
set table(ADDR20) {addr20 widget->param param->hex-widget "20MeV.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "20 MeV Address"}
set table(SS20) {ss20 int-widget->param param->int-widget "20MeV.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "20 MeV Sliding Scale"}
set table(READOUT20) {readout20 int-widget->param param->int-widget "20MeV.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "20 MeV Readout"}
set table(THRESH4) {thresh4 int-widget->param param->int-widget "4MeV.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "4 MeV Threshold"}
set table(ADDR4) {addr4 widget->param param->hex-widget "4MeV.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "4 MeV Address"}
set table(SS4) {ss4 int-widget->param param->int-widget "4MeV.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "4 MeV Sliding Scale"}
set table(READOUT4) {readout4 int-widget->param param->int-widget "4MeV.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "4 MeV Readout"}
set table(THRESHFT) {threshft int-widget->param param->int-widget "FT.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "FT TAC Threshold"}
set table(ADDRFT) {addrft widget->param param->hex-widget "FT.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "FT TAC Address"}
set table(SSFT) {ssft int-widget->param param->int-widget "FT.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "FT TAC Sliding Scale"}
set table(READOUTFT) {readoutft int-widget->param param->int-widget "FT.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "FT TAC Readout"}
set table(THRESHCO) {threshco int-widget->param param->int-widget "CO.Thresh"
       int-reg->param param->int-reg nil :slider 0 65536 1 1 "CO TAC Threshold"}
set table(ADDRCO) {addrco widget->param param->hex-widget "CO.Addr"
       int-reg->param param->int-reg nil :text 6 nil nil 1 "CO TAC Address"}
set table(SSCO) {ssco int-widget->param param->int-widget "CO.SSEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "CO TAC Sliding Scale"}
set table(READOUTCO) {readoutco int-widget->param param->int-widget "CO.ROEnable"
       int-reg->param param->int-reg nil :checkbox nil nil nil 1 "CO TAC Enable"}
set table(FTSAMPLE) { ftsample slider-widget->param param->int-widget "FTSample"
       unit-reg->param param->unit-reg ns :slider-c 0 2000 8 1 "Fast Trigger Sample (ns)"}
set table(VALSAMPLE) {valsample slider-widget->param param->int-widget "ValSample"
       unit-reg->param param->unit-reg ns :slider-c 0 10000 40 1 "Validation Sample (ns)"}
set table(WATCHDOG) {watchdog slider-widget->param param->int-widget "Watchdog"
       unit-reg->param param->unit-reg ns :slider-c 0 16000 63 1 "LT Watchdog Timeout (ns)"}
set table(PDSGATE) {pdsgate slider-widget->param param->int-widget "PDSGate"
       unit-reg->param param->unit-reg ns :slider 0 10000 40 1 "PDS Gate (ns)"}

set table(MODELCODE) {modcode str-widget->param param->hex-widget "VXI.ModelCode"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(SERIALNUM) {serialnum str-widget->param param->hex-widget "VXI.SerialNum"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MODLEVEL) {modlevel str-widget->param param->hex-widget "VXI.ModLevel"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(MANID) {manid str-widget->param param->hex-widget "VXI.ManID"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(OFFSET) {offset str-widget->param param->hex-widget "VXI.Offset"
       int-reg->param param->int-reg nil nil nil nil nil 0 nil}
set table(TESTGEN) {testgen slider-widget->param param->int-widget "TestGen" 
   unit-reg->param param->unit-reg keV :slider-c 0 7000 1 1 "Test Generator (KeV)"}
set table(ATTN) {attn int-widget->param param->int-widget "TestGen.Attn"
       int-reg->param param->int-reg nil :checkbox-c nil nil nil 1 "Test Generator Attenuation"}


# inspection tables must be in same order as entries in the :choice-stacks

set analogueinsptable {Disconnected
	4MeV-Unipolar 20MeV-Unipolar Pole-Zero
	4MeV-PDS TFA-Input TFA-Output 4MeV-Bipolar FT-TAC-Output
}


set digitalinsptable {Disconnected
	CFD-Delay Pileup PDS-Gate CFD-Direct FT-Sample Val-Sample LT-Reset
	Encode-Command
	FT-Pulse Val-Pulse EOC RO-Ack OR-RO-Ready ADC-Clock Last-Pass OR-Deadtime
}

set voltageinsptable {Disconnected
	CFD-Thresh CFD-Delay Pole-Zero PDS-Gate 
	Test-Gen CFD-Width FT-Sample Val-Sample LT-Timeout 
	DAC-Reference Vee-24 Vee-18 Vee-12 Vee-6 Vee-5.2 Vee-2 Ground
	Vcc+5-Analogue Vcc+5-Digital Vcc+6 Vcc+12 Vcc+18 Vcc+20 Vcc+24 
}

set insp_table(MUX-AI1) {"Insp.A1" AI}
set insp_table(MUX-AI2) {"Insp.A2" AI}
set insp_table(MUX-DI1) {"Insp.D1" DI}
set insp_table(MUX-DI2) {"Insp.D2" DI}
set insp_table(MUX-VI)  {"Insp.V"  VI}

set insp_menu(MUX-AI1) analogueinsptable
set insp_menu(MUX-AI2) analogueinsptable
set insp_menu(MUX-DI1) digitalinsptable
set insp_menu(MUX-DI2) digitalinsptable
set insp_menu(MUX-VI)  voltageinsptable

set inspection_widgetnames {MUX-AI1 MUX-AI2 MUX-DI1 MUX-DI2 MUX-VI}
set inspection_regnames    {Insp.A1 Insp.A2 Insp.D1 Insp.D2 Insp.V}

#    these are common
set steplist1 "CFDWIDTH FTSAMPLE VALSAMPLE WATCHDOG"
#    these are per segment
set steplist2 "CFDDELAY CFDTHRESH POLEZERO PDSGATE"

loadsource common.vxi


proc click arg {
   global debug  userselection allchans allsegs
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      REDISPLAY    {click-redisplay}
      ENGINEERING  {click-$v}
      FORMAT       {click-format}
      STEP         {click-step $v $i}

      CHANNEL {
         click-channel $v
         catch {send clover    click-channel $v}
         catch {send cloverbgo click-channel $v}
      }
      SEGMENT      {click-segment $v}

      ALLCHANS  {click-whichchans $v}
      ALLSEGS   {click-whichsegs $v}

      CHANSEL   {click-chanselection $v}
      SEGSEL    {click-segselection $v}

      PARAM        {click-param $v}
      MUX-*        {click-inspection $w [lindex $arg 4]}

      default   {
          if {$allsegs < 0 || $allchans < 0} then {set userselection 1}
             click-on $w $v
             if {$w == "TFA"} then {act_on_tfagain}
             set userselection 0
      }
   }
}

proc make-frame1 {} {
  global env chanlist seglist analogueinsptable digitalinsptable voltageinsptable stepmenu stepmode

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 5
     :layout H; :row-gap 10; :col-gap 10
      :choice-stack CHANNEL :label "Current Channel:" :strings $chanlist :valuex 18c
      :choice-stack SEGMENT :label "Current Segment:" :strings $seglist
      :checkbox ENABLE :label "Enable:" :valuex 10c
      :next-row T
      :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
      :choice ALLSEGS  :label "Act on Segments?" :valuex 18c :strings [list Current All Select] :value 0
      :next-row T
      :choice TFA  :label "TFA Gain:"  :strings [list "Low" "High"] :valuex 12c
      :hskip 100
      :choice MARKREJECT :label "Pileup & Anti-Compton:" :strings [list Mark Reject]

    :panel PANEL2 :layout T :padx 10
      :layout H; :col-gap 25
      :hskip 75
      :message M1 :label SS        :bold T
      :message M2 :label RO        :bold T
      :message M3 :label Address   :bold T
      :hskip 50
      :message M4 :label Threshold :bold T
      :row-gap 5; :next-row T
      :checkbox SS20      :valuex 12c :label "20 MeV:" 
      :checkbox READOUT20             :label " "
      :text ADDR20                    :label " " :w 6   :read-only T 
      :slider THRESH20                :label " " :min 0 :max 65535
      :row-gap 5; :next-row T
      :checkbox SS4       :valuex 12c :label "4 MeV:" 
      :checkbox READOUT4              :label " "
      :text ADDR4                     :label " " :w 6   :read-only T 
      :slider THRESH4                 :label " " :min 0 :max 65535
      :next-row T
      :checkbox SSFT      :valuex 12c :label "Fast Trigger:" 
      :checkbox READOUTFT             :label " "
      :text ADDRFT                    :label " " :w 6   :read-only T 
      :slider THRESHFT                :label " " :min 0 :max 65535
      :next-row T
      :checkbox SSCO      :valuex 12c :label "Cross Over:" 
      :checkbox READOUTCO             :label " "
      :text ADDRCO                    :label " " :w 6   :read-only T 
      :slider THRESHCO                :label " " :min 0 :max 65535


    :panel PANEL3 :layout T :padx 10 :pady 10
     :layout V; :row-gap 5
      :slider CFDWIDTH  :label "CFD Width (ns):"            :min 0 :max 200   :valuex 30c
      :slider CFDDELAY  :label "CFD Delay (ns):"            :min 0 :max 200   :valuex 30c
      :slider FTSAMPLE  :label "Fast Trigger Sample (ns):"  :min 0 :max 2000  :valuex 30c
      :slider VALSAMPLE :label "Validation Sample (ns):"    :min 0 :max 16000 :valuex 30c
      :slider WATCHDOG  :label "LT Watchdog Timeout (ns):"  :min 0 :max 16000 :valuex 30c
      :slider PDSGATE   :label "PDS Gate (ns):"             :min 0 :max 10000 :valuex 30c
      :slider CFDTHRESH :label "CFD Threshold (keV):"       :min 0 :max 2600  :valuex 30c
      :slider POLEZERO  :label "Pole Zero:"                 :min 0 :max 255   :valuex 30c

      :vskip 10
      :layout H; :col-gap 10; :row-gap 10
      :next-row T
      :menubutton MUX-AI1 :label "Analogue Inspection 1:" :w 20 :menu $analogueinsptable
      :message MUX-AI1-USE   :label ""
      :next-row T
      :menubutton MUX-AI2 :label "Analogue Inspection 2:" :w 20 :menu $analogueinsptable
      :message MUX-AI2-USE   :label ""
      :next-row T
      :menubutton MUX-DI1 :label "Digital Inspection 1:"  :w 20 :menu $digitalinsptable
      :message MUX-DI1-USE   :label ""
      :next-row T
      :menubutton MUX-DI2 :label "Digital Inspection 2:"  :w 20 :menu $digitalinsptable
      :message MUX-DI2-USE   :label ""
      :next-row T
      :menubutton MUX-VI  :label "Voltage Inspection:"    :w 20 :menu $voltageinsptable
      :message MUX-VI-USE   :label ""  

   :panel PANEL6  :layout B :padx 10 :after FOOTER
    :layout H; :col-gap 10
      :button REDISPLAY 
      :menubutton STEP      :label "[lindex $stepmenu $stepmode]"  :strings $stepmenu 
      :button FORMAT        :label "By Parameter"
      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]
      :hskip 25
      :button HELP          :command click-help
   :show T
}



proc make-frame2 {} {
  global env parameternames

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]"

        :panel PARAMS :layout T :padx 5 :pady 5
         :layout H
          :choice-stack PARAM :label "Parameter:" :valuex 15c :strings $parameternames
          :hskip 70
          :choice ALLCHANS :label "Act on Channels?" :valuex 18c :strings [list Current All Select] :value 0
          :choice ALLSEGS  :label "Act on Segments?" :valuex 18c :strings [list Current All Select] :value 0

        :panel PANEL :layout T :padx 5

        :panel BUTTONS :layout B :padx 5 :after FOOTER
         :layout H
          :button REDISPLAY
          :button FORMAT       :label "By Channel"
          :hskip 80
          :button HELP         :command click-help

  :show T

}


global electronicsframe
set electronicsframe [self frame-name].electronics

proc make-electronics {} {
  global env electronicsframe electronicsvisible

  :frame $electronicsframe :label "Clover Ge Electronics" :resize F :transient T

    :panel ELECTRONICS-PANEL :layout T :padx 5 :pady 5
     :layout H; :row-gap 5
      :checkbox LASTCARD  :label "Last Card:"                            :valuex 20c
      :text MODELCODE     :label "Model Code:" :w 6  :read-only T        :valuex 20c
      :next-row T
      :checkbox MODE      :label "Test Mode:"                            :valuex 20c
      :text SERIALNUM     :label "Serial Number:" :w 6 :read-only T      :valuex 20c
      :next-row T
      :checkbox CONFENETRES :label "Controle Fenetres:"                  :valuex 20c
      :text MODLEVEL      :label "Modification Level:" :w 6 :read-only T :valuex 20c
      :next-row T
      :text MANID         :label "Manufacturer ID:" :w 6  :read-only T   :valuex 20c
      :next-row T
      :checkbox ATTN      :label "Test Gen Attenuation:"                 :valuex 20c
      :text OFFSET        :label "Offset Register:" :w 6 :read-only T    :valuex 20c
      :row-gap 20; :next-row T; :row-gap 10
      :choice TEST-GE-A    :exclusive F :strings "A" :label "Ge Test Mode:" :w 3
      foreach i {B C D E} {
        :choice TEST-GE-$i :exclusive F :strings "$i" :w 3
      }
      :next-row T
      :slider TESTGEN     :label "Test Generator (keV):" :min 0 :max 7000 :valuex 20c
  
      :show T
   set electronicsvisible 1
}        


proc make-segSELframe {} {
  global segsel seglist

   :frame SEGSEL-FRAME :label "Clover Ge Segment Selection"  :show-footer F :resize F :transient T
   :panel SEGSEL-PANEL :layout T
   :choice SEGSEL :exclusive F :strings $seglist :value $segsel
   :show T
}         

proc make-chanSELframe {} {
  global chansel chanlist

   :frame CHANSEL-FRAME :label "Clover Ge Channel Selection"  :show-footer F :resize F :transient T
   :panel CHANSEL-PANEL :layout T
   :choice CHANSEL :exclusive F :strings $chanlist :value $chansel
   :show T
}         



