

#   clusterbgo.defn.tcl     Euroball III Cluster BGO detector VXI card


proc {AddClusterBGOModule}  {module slot}  {
   global registerbase cardtype

   set registerbase $cardtype$module

   delete-vxi-registers $registerbase.*

   ClusterBGOVXIRegisters $slot
   ClusterBGOModuleRegisters  $slot

   ClusterBGORegisters $module $slot
}

proc ClusterBGORegisters {module slot} {
   global registerbase cardtype

   set registerbase $cardtype$module.Global
   ClusterBGOVXIRegisters $slot
   ClusterBGOModuleRegisters  $slot

#    registers for SAROC 1 at offset 0x100 channel 0   -   bgoPattern

   set registerbase $cardtype$module.BGOPattern
   ClusterBGOSAROCPatternRegisters $slot 0 0x100

#    registers for SAROC 4 at offset 0x280 channel 4  -  bgo Sum

   set registerbase $cardtype$module.BGOSum
   ClusterBGOSAROCRegisters $slot 4 0x280

#    channel area registers   -   20th channel used for Sum

   set registerbase $cardtype$module.BGOSum
   ClusterBGODetectorRegisters $slot 19

#    registers for SAROC 1 at offset 0x100 channel 1 =>     BgoA => BgoE

   set c 1
   foreach segment {A B C D E} {
      set registerbase $cardtype$module.BGO$segment
      ClusterBGOSAROCRegisters $slot $c 0x100
      incr c
   }

#    registers for SAROC 2 at offset 0x180 channel 0 =>     BgoF => BgoJ

   set c 0
   foreach segment {F G H I J} {
      set registerbase $cardtype$module.BGO$segment
      ClusterBGOSAROCRegisters $slot $c 0x180
      incr c
   }

#    registers for SAROC 3 at offset 0x200 channel 0 =>     BgoK => BgoO

   set c 0
   foreach segment {K L M N O} {
      set registerbase $cardtype$module.BGO$segment
      ClusterBGOSAROCRegisters $slot $c 0x200
      incr c
   }

#    registers for SAROC 4 at offset 0x280 channel 0 =>     BgoP => BgoS

   set c 0
   foreach segment {P Q R S} {
      set registerbase $cardtype$module.BGO$segment
      ClusterBGOSAROCRegisters $slot $c 0x280
      incr c
   }

#    channel area registers   -  first 19 channels

   set c 0
   foreach segment {A B C D E F G H I J K L M N O P Q R S} {
      set registerbase $cardtype$module.BGO$segment
      ClusterBGODetectorRegisters  $slot $c 
      incr c
   }
}

proc clusterbgo_register {registername class attributes} {
   global registerbase
   define-vxi-register $registerbase $class $registername "$attributes"
}


proc ClusterBGOVXIRegisters {slot} {

   clusterbgo_register  datal          xDig "-s$slot  -o0x0 -w32 -prw -fx"
   clusterbgo_register  dataw          xDig "-s$slot  -o0x0 -w16 -prw -fx"

   clusterbgo_register  VXI.ManID      xSht  "-s$slot -o0  -pro -w16 -fx -b0  -l12"
   clusterbgo_register  VXI.AddrSpace  xSht  "-s$slot -o0  -pro -w16 -fd -b12 -l2"
   clusterbgo_register  VXI.DevClass   xSht  "-s$slot -o0  -pro -w16 -fd -b14 -l2"
   clusterbgo_register  VXI.ModelCode  xSht  "-s$slot -o2  -pro -w16 -fx -b0  -l12"
   clusterbgo_register  VXI.ReqMem     xSht  "-s$slot -o2  -pro -w16 -fd -b12 -l4"
   clusterbgo_register  VXI.Reset      xSht  "-s$slot -o4  -prw -w16 -fd -b0  -l1"
   clusterbgo_register  VXI.Passed     xSht  "-s$slot -o4  -pro -w16 -fd -b2  -l1"
   clusterbgo_register  VXI.Ready      xSht  "-s$slot -o4  -pro -w16 -fd -b3  -l1"
   clusterbgo_register  VXI.ModID      xSht  "-s$slot -o4  -pro -w16 -fd -b14 -l1"
   clusterbgo_register  VXI.A24A32     xSht  "-s$slot -o4  -pro -w16 -fd -b15 -l1"
   clusterbgo_register  VXI.Offset     xSht  "-s$slot -o6  -pro -w16 -fx"
   clusterbgo_register  VXI.SerialNum  xSht  "-s$slot -o8  -pro -w16 -fx"
   clusterbgo_register  VXI.ModLevel   xSht  "-s$slot -o10 -pro -w16 -fx"
}


proc ClusterBGOModuleRegisters {slot} {
   global registerbase

#    common area registers

        clusterbgo_register MCR               xDig "-s$slot -o0x10 -w16 -prw -fx -b0 -l0    "
        clusterbgo_register SSCEnable         xDig "-s$slot -o0x10 -w16 -prw -fd -b2 -l1 -i0"
        clusterbgo_register SSDACEnable       xDig "-s$slot -o0x10 -w16 -prw -fd -b3 -l1 -i0"
        clusterbgo_register ACPWEnable        xDig "-s$slot -o0x10 -w16 -prw -fd -b4 -l1 -i0"
        clusterbgo_register MCR-b5            xDig "-s$slot -o0x10 -w16 -prw -fd -b5 -l1    "
        clusterbgo_register MCR-b6            xDig "-s$slot -o0x10 -w16 -prw -fd -b6 -l1    "
        clusterbgo_register MCR-b7            xDig "-s$slot -o0x10 -w16 -prw -fd -b7 -l1    "
        clusterbgo_register FIFO.Reset        xDig "-s$slot -o0x10 -w16 -prw -fd -b8 -l1 -i1"
        clusterbgo_register GIR.Reset         xDig "-s$slot -o0x10 -w16 -prw -fd -b9 -l1    "
        clusterbgo_register LastCard          xDig "-s$slot -o0x10 -w16 -prw -fd -b10 -l1   "
        clusterbgo_register MCR-b11           xDig "-s$slot -o0x10 -w16 -prw -fd -b11 -l1    "
        clusterbgo_register MCR-b12           xDig "-s$slot -o0x10 -w16 -prw -fd -b12 -l1    "
        clusterbgo_register MCR-b13           xDig "-s$slot -o0x10 -w16 -prw -fd -b13 -l1    "
        clusterbgo_register DSP.Reset         xDig "-s$slot -o0x10 -w16 -prw -fd -b14 -l1 -i0"
        clusterbgo_register MCR-b15           xDig "-s$slot -o0x10 -w16 -prw -fd -b15 -l1    "

#    for the Cluster BGO card the signal register is assigned class xMux because this must be 
#    set to zero to discounnect the inspection line and NOT the channel register

        clusterbgo_register Insp.D1.Channel   xDig "-s$slot -o0x20 -w16 -prw -fd"
        clusterbgo_register Insp.D1.Signal    xMux "-s$slot -o0x22 -w16 -prw -b0 -l16  -dCluster -md1"
        clusterbgo_register Insp.D2.Channel   xDig "-s$slot -o0x24 -w16 -prw -fd"
        clusterbgo_register Insp.D2.Signal    xMux "-s$slot -o0x26 -w16 -prw -b0 -l16  -dCluster -md2"
        clusterbgo_register Insp.A1.Channel   xDig "-s$slot -o0x28 -w16 -prw -fd"
        clusterbgo_register Insp.A1.Signal    xMux "-s$slot -o0x2a -w16 -prw -b0 -l16  -dCluster -ma1"
        clusterbgo_register Insp.A2.Channel   xDig "-s$slot -o0x2c -w16 -prw -fd"
        clusterbgo_register Insp.A2.Signal    xMux "-s$slot -o0x2e -w16 -prw -b0 -l16  -dCluster -ma2"
        clusterbgo_register Insp.V.Channel    xDig "-s$slot -o0x30 -w16 -prw -fd"
        clusterbgo_register Insp.V.Signal     xMux "-s$slot -o0x32 -w16 -prw -b0 -l16  -dCluster -mv"

        clusterbgo_register SAROC1.Test       xDig "-s$slot -o0x160 -w32 -pwo -fx -b0 -l0"
        clusterbgo_register SAROC1.Offset     xDig "-s$slot -o0x164 -w32 -pro -fx -b0 -l0"
        clusterbgo_register SAROC2.Test       xDig "-s$slot -o0x1e0 -w32 -pwo -fx -b0 -l0"
        clusterbgo_register SAROC2.Offset     xDig "-s$slot -o0x1e4 -w32 -pro -fx -b0 -l0"

#    common area DACs

        clusterbgo_register   FTSample     xDAC "-s$slot -c0 -a0x2400 -z0x0 -o0x0 -w16 -pwo -qn -us -d0 -r0:0s,0xff:3.2us -i400ns"
        clusterbgo_register   ValSample    xDAC "-s$slot -c0 -a0x2500 -z0x0 -o0x0 -w16 -pwo -qn -us -d0 -r0:0s,0xff:16us  -i5000ns"
        clusterbgo_register   VETO         xDAC "-s$slot -c0 -a0x2600 -z0x0 -o0x0 -w16 -pwo -qn -us -d0 -r0:0s,0xff:16us  -i8000ns"
        clusterbgo_register   SpareDac     xDAC "-s$slot -c0 -a0x2700 -z0x0 -o0x0 -w16 -pwo -qn -us -d0 -r0:0s,0xff:25us  -i12500ns"


#    GIR registers

        clusterbgo_register GIR.EvNum.Count    xDig "-s$slot -o0xA0 -w16 -prw -fx -b0 -l0 -i0"
        clusterbgo_register GIR.EvNum.Addr     xAdr "-s$slot -c0 -o0xA2 -w16 -b0 -l0 -i$registerbase.GIR.EvNum"
        clusterbgo_register GIR.EvNum          xDig "-s$slot -o0xA0 -w32 -pro -fx -b0 -l0    "
        clusterbgo_register GIR.FIFO.Test      xFIT "-s$slot -o0xA4 -w32 -prw -fx"

        clusterbgo_register GIR.ROTSR          xDig "-s$slot -o0xAC -w16 -pro -fx -b0 -l0    "
        clusterbgo_register GIR.ValAck.TimeOut xDAC "-s$slot -c0 -a0 -z0 -o0xAE -w16 -prw -qn -us -d0 -r0:0s,0xffff:4096000ns -i75000ns"
        clusterbgo_register GIR.Qstat0         xDig "-s$slot -o0xB0 -w32 -pro -fd -b0 -l16    "
        clusterbgo_register GIR.Qstat1         xDig "-s$slot -o0xB0 -w32 -pro -fd -b16 -l16    "

        clusterbgo_register GIR.ROCR                      xDig "-s$slot -o0xA8 -w16 -prw -fx -b0 -l0 -i0"
        clusterbgo_register GIR.ROCR.SAROC.Test           xDig "-s$slot -o0xA8 -w16 -prw -fx -b0 -l1    "
        clusterbgo_register GIR.ROCR.Ext.Test             xDig "-s$slot -o0xA8 -w16 -prw -fx -b1 -l1    "
        clusterbgo_register GIR.ROCR.ValAck.TimeOutReset  xDig "-s$slot -o0xA8 -w16 -prw -fx -b2 -l1 -i1"
        clusterbgo_register GIR.ROCR.ReadOut.Disable      xDig "-s$slot -o0xA8 -w16 -prw -fx -b3 -l1    "
        clusterbgo_register GIR.ROCR.PipeLine.Enable      xDig "-s$slot -o0xA8 -w16 -prw -fx -b5 -l1    "
        clusterbgo_register GIR.ROCR.EvNumData.Disable    xDig "-s$slot -o0xA8 -w16 -prw -fx -b6 -l1 -i1"
        clusterbgo_register GIR.ROCR.ReadOutMode          xDig "-s$slot -o0xA8 -w16 -prw -fx -b7 -l1 -i0"
        clusterbgo_register GIR.ROCR.DSPEN                xDig "-s$slot -o0xA8 -w16 -prw -fx -b8 -l1"
	clusterbgo_register GIR.ROCR.MODEDMA              xDig "-s$slot -o0xA8 -w16 -prw -fx -b9 -l1"
        clusterbgo_register GIR.ROCR.EvNumQbits.Enable    xDig "-s$slot -o0xA8 -w16 -prw -fx -b11 -l1 -i1"

        clusterbgo_register GIR.ROTSR.Temp.Status            xDig "-s$slot -o0xAC -w16 -pro -fx -b0 -l2 "
        clusterbgo_register GIR.ROTSR.ValAck.TimeOut.Status  xDig "-s$slot -o0xAC -w16 -pro -fx -b6 -l1 "
        clusterbgo_register GIR.ROTSR.FIFO.Status            xDig "-s$slot -o0xAC -w16 -pro -fx -b8 -l2 "
        clusterbgo_register GIR.ROTSR.SAROC.Status           xDig "-s$slot -o0xAC -w16 -pro -fx -b12 -l1 "
        clusterbgo_register GIR.ROTSR.EvFault.Status         xDig "-s$slot -o0xAC -w16 -pro -fx -b13 -l3 "

#    GIR DSP Registers

        clusterbgo_register GIR.DSP.Control         xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b0 -l0"
        clusterbgo_register GIR.DSP.StartDSP        xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b16 -l1"
        clusterbgo_register GIR.DSP.StopDSP         xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b18 -l1"
        clusterbgo_register GIR.DSP.ResetDSP        xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b20 -l1"
        clusterbgo_register GIR.DSP.Mux        xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b21 -l1"
        clusterbgo_register GIR.DSP.Pz        xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b22 -l1"
        clusterbgo_register GIR.DSP.ResetMemExt        xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b23 -l1"
        clusterbgo_register GIR.DSP.Control.FilterMode   xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b17 -l1"
        clusterbgo_register GIR.DSP.Control.DSPtotcount  xDig "-s$slot -a0x400000 -o0xbff40 -w32 -prw -fd -b19 -l1"
	clusterbgo_register   GIR.DSP.Spectre     xDig    "-s$slot  -a0x400000 -o0xbff40 -w32 -prw -fd -b24 -l1"
	clusterbgo_register   GIR.DSP.Scaler     xDig    "-s$slot  -a0x400000 -o0xbff40 -w32 -prw -fd -b25 -l1"
        clusterbgo_register GIR.DSP.Mirror          xDig "-s$slot -a0x400000 -o0xbff44 -w32 -prw -fx -b16 -l1"
        clusterbgo_register GIR.DSP.Item            xDig "-s$slot -a0x400000 -o0x98004 -w32 -prw -fx -b0 -l0"

        clusterbgo_register GIR.DSP.Program          xGIR "-s$slot -a0x400010"
        clusterbgo_register GIR.DSP.ExtMem           bVXI "-s$slot -a0x200000 -o0x0 -w32 -pro -fx -l64"
        clusterbgo_register GIR.DSP.SpectrumData     bVXI "-s$slot -a0x200000 -o0x0 -w32 -pro -fx -l64"
        clusterbgo_register GIR.DSP.IntMem           bVXI "-s$slot -a0x400000 -o0x0 -w32 -pro -fx -l64"

}

proc ClusterBGODetectorRegisters {slot c} {

        clusterbgo_register   DiscThresh     xDAC "-s$slot -c$c -a0x1000 -z0x100 -o0x0 -w16 -pwo -qm -uV -d0 -r0:63mV,0xff:0mV -i32mV"
        clusterbgo_register   CCR            xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x2 -w16 -prw -fx -b0 -l0"
        clusterbgo_register   Enable         xDig "-s$slot -c$c -a0x1000 -z0x100 -o0x2 -w16 -prw -fd -b0 -l2 -i0"
}

proc ClusterBGOSAROCRegisters {slot c a} {
   global registerbase

    clusterbgo_register Energy.Addr       xAdr "-s$slot -c$c -a$a -z16 -o0x00 -w32 -i$registerbase.Energy"
    clusterbgo_register Energy.TestData   xDig "-s$slot -c$c -a$a -z16 -o0x00 -w32 -g0 -prw -fx -b16 -l0"
    clusterbgo_register Energy.TestRd     xDig "-s$slot -c$c -a$a -z16 -o0x00 -w32 -g0 -pro -fx -b16 -l0"
    clusterbgo_register Energy.Thresh     xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b0  -l16 -i0x300"
    clusterbgo_register Energy.ShiftCtrl  xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b16 -l2 -i2"
    clusterbgo_register Energy.ROEnable   xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterbgo_register Energy.SSEnable   xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b19 -l1 -i1"
    clusterbgo_register Energy.OSDisable  xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b20 -l1 -i0"

    clusterbgo_register TAC.Addr       xAdr "-s$slot -c$c -a$a -z16 -o0x08 -w32 -i$registerbase.TAC"
    clusterbgo_register TAC.TestData   xDig "-s$slot -c$c -a$a -z16 -o0x08 -w32 -g0 -prw -fx -b16 -l0"
    clusterbgo_register TAC.TestRd     xDig "-s$slot -c$c -a$a -z16 -o0x08 -w32 -g0 -pro -fx -b16 -l0"
    clusterbgo_register TAC.Thresh     xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b0  -l16 -i0x400"
    clusterbgo_register TAC.ShiftCtrl  xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b16 -l2 -i2"
    clusterbgo_register TAC.ROEnable   xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterbgo_register TAC.SSEnable   xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b19 -l1 -i1"
    clusterbgo_register TAC.OSDisable  xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b20 -l1 -i0"
}

proc ClusterBGOSAROCPatternRegisters {slot c a} {
   global registerbase

    clusterbgo_register High.Addr       xAdr "-s$slot -c$c -a$a -z16 -o0x00 -w32 -i$registerbase.High"
    clusterbgo_register High.TestData   xDig "-s$slot -c$c -a$a -z16 -o0x00 -w32 -g0 -prw -fx -b16 -l0"
    clusterbgo_register High.TestRd     xDig "-s$slot -c$c -a$a -z16 -o0x00 -w32 -g0 -pro -fx -b16 -l0"
    clusterbgo_register High.Thresh     xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b0  -l16 -i0"
    clusterbgo_register High.ShiftCtrl  xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b16 -l2 -i0"
    clusterbgo_register High.ROEnable   xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterbgo_register High.SSEnable   xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b19 -l1 -i0"
    clusterbgo_register High.OSDisable  xDig "-s$slot -c$c -a$a -z16 -o0x04 -w32 -g1 -prw -fd -b20 -l1 -i1"

    clusterbgo_register Low.Addr       xAdr "-s$slot -c$c -a$a -z16 -o0x08 -w32 -i$registerbase.Low"
    clusterbgo_register Low.TestData   xDig "-s$slot -c$c -a$a -z16 -o0x08 -w32 -g0 -prw -fx -b16 -l0"
    clusterbgo_register Low.TestRd     xDig "-s$slot -c$c -a$a -z16 -o0x08 -w32 -g0 -pro -fx -b16 -l0"
    clusterbgo_register Low.Thresh     xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b0  -l16 -i0"
    clusterbgo_register Low.ShiftCtrl  xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b16 -l2 -i0"
    clusterbgo_register Low.ROEnable   xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b18 -l1 -i1"
    clusterbgo_register Low.SSEnable   xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b19 -l1 -i0"
    clusterbgo_register Low.OSDisable  xDig "-s$slot -c$c -a$a -z16 -o0x0c -w32 -g1 -prw -fd -b20 -l1 -i1"
}




proc ClusterBGO_DAQ-reset   {args}   {
   set module [first $args]
#    flush FIFOs
   write-vxi-register $module.FIFO.Reset 0
   write-vxi-register $module.FIFO.Reset 1
   write-vxi-register $module.FIFO.Reset 0
#    reset and enable GIR
   write-vxi-register $module.GIR.FIFO.Test 0
   read-vxi-register $module.GIR.FIFO.Test
}


proc ClusterBGO_DAQ-halt   {args}   {
   set module [first $args]
#    no action
}


proc ClusterBGO_DAQ-go   {args}   {
   set module [first $args]
#    zero event numbers
   write-vxi-register $module.GIR.EvNum.Count 0
}

