#    Diagnostic task for testing VXI modules

set frame VXIDIAG
set title "VXI Diagnostics"
set icon VXIDiag

set started 0
set Testing 0
set stage 0

#    by default rdw and rdl procs write to the log window
set LogFlag 1

if {[info exists env(VXIServer)] == 0} then {set VXIServer VXI}  \
   else {set VXIServer $env(VXIServer)}

set DiagFileBase [file join $env(MIDASBASE) vxi tcl diag-scripts]

set server "xxxx"
set serverindex 0
set VXICrateList {}
set VXIHostList {}
set slot -1

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set histlim 40
for {set i 0} {$i <= $histlim} {incr i} {lappend history ""}
set nextcommand $histlim


proc make-frame {} {
  global env VXICrateList serverindex stage

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]" 

   :panel P1 :layout T :padx 10 :pady 10
     :layout H; :col-gap 10; :row-gap 10
       :choice-stack SERVER :label "Server choice" :menu $VXICrateList :value $serverindex
       :button START     :w 10 :command start_diag
       :button FINISH    :w 10 :command click-quit

   set stage 0

   :show T
}

proc click arg {
   global history
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit $f]}
   clear-footer
   switch -glob $w  {

      CMD        {invoke "$v"}
      HISTORY    {
                 set i [first $v]
                 invoke "[lindex $history $i]"
                 }

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

#    inital application start - just builds the initial frame containing the server
#    choice
proc start args {
   global started server VXICrateList serverindex

#    built the choice menu for the server
   make-serverlist
   set server [lindex $VXICrateList $serverindex]

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
}

proc click-trace {v} {
    global mname
    set mname VXI
    write-register TraceMode $v
    :modify TRACE :value [read-register TraceMode]
}

proc click-server {v} {
    global stage server VXICrateList serverindex slot Testing
    global EGallow mname rmAddress rmShortAddress 

    set serverindex $v
    set server [lindex $VXICrateList $serverindex]

    if {$stage == 0} then {return}

    set-footer "Connecting to server $server"
    define-SERVER-registers
#    obtain base addresses allocated by the RM to all installed modules
    for {set i 0} {$i <= 12} {incr i} {
      set mname "Address"
      set addresses [read-register s$i]
      set rmAddress($i) [first $addresses]
      set rmShortAddress($i) [second $addresses]
   }
    clear-footer
#    report status of slots in the crate
    log "Connected to server $server"
    for {set i 0} {$i <= 12} {incr i} {
       if {$rmAddress($i) == 0} then {
           log "slot $i: empty"
       } else {
           log "slot $i: short i/o address $rmShortAddress($i)"
           log "slot $i: base address $rmAddress($i)"
           set mname Sht
           foreach offset {0 2 4 6} {
              modify-register s$i "-o $offset"
              set VXIdata($i,$offset) [read-register s$i]
           }
           log "slot $i config regs: $VXIdata($i,0) $VXIdata($i,2) $VXIdata($i,4) $VXIdata($i,6)"
       }
    }

#     select slot containing first VXI card found
    for {set i 1} {$i <= 12} {incr i} {if {$rmAddress($i) != 0} then {break}}
    if {$i > 12} then {
       if {$Testing == 0} then {
          midas-warning "No VXI card found in the crate $server"
          set i 0
       } else {
          set i 1
          set rmAddress(1) 0x00008000
          set rmShortAddress(1) 0xffffc000
       }
    }
    set slot $i
    :modify SLOT :value $slot

    click-slot  $slot

    set mname VXI
#    :modify TRACE :value [read-register TraceMode]
}


#    starts the task in full once a server has been selected
proc start_diag {} {
   global env stage VXICrateList serverindex

  set rows 8
  set cols 50

  set stage 1

  :set autoreturn "F"

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/vxi_module" \
      :icon-name "[self icon-title]" :resize T

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :col-gap 10; :row-gap 10
       :choice-stack SERVER :label "Server choice" :menu $VXICrateList :value $serverindex
       :button HELP      :command click-help
       :button FINISH    :command click-quit

   :panel P2 :layout T :padx 10 :pady 5
     :layout H; :col-gap 25; :row-gap 10

     :button BAT :label "Execute file"
     :checkbox TRACE :label "VXI crate TraceMode" 
     :next-row T
     :layout V
     :message SLOTM  :label "VXI crate slot choice"
     :choice SLOT    :label ""    :strings [list 0 1 2 3 4 5 6 7 8 9 10 11 12] :w 2

   :panel P3 :layout T :padx 10
     :galley LOG :rows $rows :cols $cols :font variable
     configuretags

   :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 10; :row-gap 10
     :button EMPTYLOG :label "Empty program log window" :command ":modify LOG :empty T"
     :menubutton HISTORY   :label "History"  :strings [list ""]
     :next-row T
     :colour local entry foreground black
     :text CMD  :label "command:" :w [expr $cols - 10]
    bind [:path CMD].entry <Meta-k>         "restorecommand last"
    bind [:path CMD].entry <Meta-BackSpace> "zapcommand"
    bind [:path CMD].entry <Alt-KeyPress-k>         "restorecommand last"
    bind [:path CMD].entry <Alt-KeyPress-BackSpace> "zapcommand"
    bind [:path CMD].entry <Up> "restorecommand previous"
    bind [:path CMD].entry <Down> "restorecommand next"

     open-frame

     :galley_size LOG $rows $cols

    focus [:path CMD].entry

#    finally contact the server
     click-server $serverindex
}

proc click-configure {W} {:galley_resize LOG}

proc restorecommand {c} {
  global history histlim nextcommand 
  switch $c {
    last  {:modify CMD :value [lindex $history end]}
    previous  {
     :modify CMD :value [lindex $history $nextcommand]
     incr nextcommand -1
     if {$nextcommand < 0} then {set nextcommand 0}
    }
    next  {
     incr nextcommand 1
     if {$nextcommand > $histlim} then {set nextcommand $histlim}
     :modify CMD :value [lindex $history $nextcommand]
    }
    default {log "restorecommand $c?"}
  }
}

proc zapcommand {} {:modify CMD :value ""}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc click-bat args {
    global env slot server DiagFileBase

#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{Command Files}      {.bat}   }
      {{All Files}   *        }
    }
    set batfile [tk_getOpenFile -parent [self frame-name] -title "Diagnostic script Selection"  \
       -initialdir $DiagFileBase -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$batfile != ""} then {
       set directory [file dirname $batfile]
       set file [file tail $batfile]
       set z [midas-confirmation "Execute file $file for the VXI card
          in slot $slot of VXI crate $server"]
       if {$z == 0} then {return}
       uplevel #0 source $batfile
       set DiagFileBase $directory
    }
}

proc click-slot {v} {
   global server slot rmAddress
   if {$rmAddress($v) == 0} then {
      set-footer "There is no card in slot $v"
   } else {
      set slot $v
      set-footer "slot $slot in crate $server selected"
   }
   :modify SLOT :value $slot
}

proc invoke {text} {
    global outputprompt inputprompt
    global history histmenu histlim nextcommand

    set cmd ""
    foreach item [split $text ,] {append cmd "$item" " "}

    :modify LOG :usingtag taginput :strings "[set inputprompt]$cmd\n"

    if [info complete $cmd] {
	if {$cmd == "!!"} {
	    set cmd [lindex $history end]
	} else {
            lappend history "$cmd"
            set history [lreplace $history 0 0]
	}
	set result [catch {eval $cmd} msg]
	if {$result != 0} {
	     :modify LOG :usingtag tagoutput :strings "[set outputprompt]Error: $msg\n"
	} else {
	    if {$msg != ""} {
                :modify LOG :usingtag tagoutput :strings "$outputprompt$msg\n"
	    }
	}
    }

    :modify CMD :value ""
    set nextcommand $histlim

    set histmenu {}
    set i 0
    foreach s $history {
       if {$s != ""} then {lappend histmenu "$i [string range "$s" 0 40]"}
       incr i
    }
    :modify HISTORY :strings $histmenu
}

proc log {s} {
  global insertprompt
  :modify LOG :usingtag taginsert :strings "$insertprompt$s\n"
}

#    register access procedures

proc define-SERVER-registers {} {
   global server mname EGreport

   set EGreport -1

   set mname Address
   delete-register *

   set mname Sht
   delete-register *

   set mname Ext
   delete-register *

#    define registers which return base address (VME) for each slot
#     and allow access to the Short I/O and Extended address space
   for {set i 0} {$i <= 12} {incr i} {
      set mname Address
      define-register xAddress s$i "-s $i"
      set mname Sht
      define-register xSht s$i "-s$i -o0x0 -prw -w16 -fx"
      set mname Ext
      define-register xDig s$i "-s$i -c0 -o0x0 -prw -w32 -fx"
      set mname s$i
      set register /$server/s$i.*
      set-footer "deleting register $register"
      EG "register :pattern '$register' !delete"
      define-register xDig datal "-s$i -c0 -o0x0 -prw -w32 -fx"
      define-register xDig dataw "-s$i -c0 -o0x0 -prw -w16 -fx"
   }

   set mname VXI
#   delete-register TraceMode
#   define-register generic TraceMode "-n TraceMode -t int"

   set EGreport 0
   midas-warning " VXI crate access created - If the VXI crate is rebooted you will have to \
       reselect the VXI Crate Server"
}

proc define-register {class name attributes} {
   global server mname

   set register /$server/$mname.$name
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc delete-register {name} {
   global server mname

   set register /$server/$mname.$name
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"
}

proc modify-register {name attributes} {
   global server mname

   set register /$server/$mname.$name
   EG "register :name '$register' :object '$attributes'"
}

proc read-register {name} {
   global server mname
   set register /$server/$mname.$name
   return [EG "register :name '$register' ?value"]
}

proc write-register {name value} {
   global server mname
   set register /$server/$mname.$name
   return [EG "register :name '$register' :value '$value'"]
}

#    proc to read 32 bits from module
#    address is VME address relative to module base
proc rdl {address} {return [rd l $address]}

#    proc to read 16 bits from module
#    address is VME address relative to module base
proc rdw {address} {return [rd w $address]}

proc rd {width address} {
   global server slot LogFlag
   set a [expr $address]
   set register /$server/s$slot.data$width
   set attributes "-o [format "0x%04x" $a]"
   EG "register :name '$register' :object '$attributes'"
   set r [EG "register :name '$register' ?value"]
   if {$LogFlag == 1} then {log "slot $slot: rd$width [format "0x%04x" $a]: $r"}
   return $r
}

proc shtrd {address} {
   global server slot LogFlag
   set a [expr $address]
   set register /$server/Sht.s$slot
   set attributes "-o [format "0x%04x" $a]"
   EG "register :name '$register' :object '$attributes'"
   set r [EG "register :name '$register' ?value"]
   if {$LogFlag == 1} then {log "slot $slot: shtrd [format "0x%04x" $a]: $r"}
   return $r
}

proc rdblk {width length address} {
   global server slot
   set a [expr $address]
   set register /$server/s$slot.data$width
   set attributes "-o [format "0x%04x" $a] -l $length"
   EG "register :name '$register' :object '$attributes'"
   return [EG "register :name '$register' ?value"]
}

#    proc to write 32 bits to module
#    address is VME address relative to module base
proc wrl {address args} {
   switch [llength $args] {
       0  {set r [wr l $address 0 0]}
       1  {set r [wr l $address [first $args] 0]}
       2  {set r [wr l $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrl -  $args"; return}
   }
}

#    proc to write 16 bits to module
#    address is VME address relative to module base
proc wrw {address args} {
   switch [llength $args] {
       0  {set r [wr w $address 0 0]}
       1  {set r [wr w $address [first $args] 0]}
       2  {set r [wr w $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrw -  $args"; return}
   }
}

proc wr {width address data flag} {
   global server slot LogFlag
   set a [expr $address]
   set vl [expr $data]
   set register /$server/s$slot.data$width
   if {$LogFlag == 1} then {log "slot $slot: wr$width [format "0x%04x" $a]: $vl"}
   set attributes "-o [format "0x%04x" $a]"
   EG "register :name '$register' :object '$attributes'"
   EG "register :name '$register' :value '$vl'"
   if {$flag == 0} then {return [EG "register :name '$register' ?value"]} else {return $data}
}

proc shtwr {address data flag} {
   global server slot LogFlag
   set a [expr $address]
   set vl [expr $data]
   set register /$server/Sht.s$slot
   if {$LogFlag == 1} then {log "slot $slot: shtwr [format "0x%04x" $a]: $vl"}
   set attributes "-o [format "0x%04x" $a]"
   EG "register :name '$register' :object '$attributes'"
   EG "register :name '$register' :value '$vl'"
   if {$flag == 0} then {return [EG "register :name '$register' ?value"]} else {return $data}
}


#    procedures taken from the vxicontrol task to obtain list of
#    available VXI servers from the experiment file

proc make-serverlist {} {
  global VXICrateList VXIHostList VXIServer

#    find VXI crates available

   if {[catch {eg "(experiment ?resources)"} m]} then {
      midas-warning "no experiment connected!!!"
      set VXICrateList {}
      set VXIHostList {}
   } else {
      set VXICrateList [make-resource-list "name" $m $VXIServer]
      set VXIHostList  [make-resource-list "host" $m $VXIServer]
   }
#    sort lists into sensible order
   set cratelist [lsort $VXICrateList]
   set hostlist {}
   foreach c $cratelist {
      lappend hostlist [lindex $VXIHostList [lsearch $VXICrateList $c]]
   }
   if {[llength $VXIHostList] == 0} then {
      midas-warning "no VXI crates found in current experiment!!!"
      set VXICrateList {vxi1}
      set VXIHostList {vxi1}
   } else {
      set VXICrateList $cratelist
      set VXIHostList $hostlist
   }
}

proc make-resource-list {type rlist resource} {
   set ll {}
   foreach item $rlist {
      set l [find-resource $type $item $resource]
      if {$l!={}} then {lappend ll $l}
   }
   return $ll
}

proc find-resource {type ps resource} {
   set l {}
   set role {}; set name {}; set host {}
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         host {set host [second $p]}
         default {}
      }
   }
   if {$role==$resource}  then  {set l [set $type]}
   return $l
}

proc taskDelay {t} {after $t}

if {$Testing == 1} then {
proc EG {args} {log "EG $args"; return 1}
}
